/*
 * 名前
 *    03kuku - 九九練習プログラム
 * 書式
 *    03kuku
 * 解説
 *    九九の問題をランダムに表示してユーザに入力を促し、
 *    その正解数と正解率を表示する。
 * 作者
 *    結城浩
 *    Copyright (C) 1993, 2018 by Hiroshi Yuki.
 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#define MAX_Q 10 // 表示する問題の個数
#define BUFFER_SIZE 256 // 入力行の最大長

// プロトタイプ宣言
int main(void);
int kuku(int qn);
void get_line(char *buffer, int size);

// 問題をMAX_Q回繰り返し、最後に結果を表示する。
int main(void)
{
    double good_rate; // 正答率
    int good_answers = 0; // 正答数合計

    // 現在時刻を使って乱数の「種」を設定する。
    srand((unsigned int)time(NULL));

    // オープニングメッセージを表示する。
    printf("これから九九の問題を %d 問出します。\n", MAX_Q);

    good_answers = 0;
    for (int i = 0; i < MAX_Q; i++) {
        good_answers += kuku(i);
    }

    // 正答率を計算する。
    good_rate = good_answers * 100.0 / MAX_Q;

    // 結果を表示する。
    printf("問題は %d 問ありました。\n", MAX_Q);
    printf("%d 問は正しく答えられましたが、\n", good_answers);
    printf("%d 問はまちがってしまいました。\n", MAX_Q - good_answers);
    printf("正答率 %0.1f %%です。\n", good_rate);    
    printf("\n");
    printf("お疲れさま。\n");
    return 0;
}

// 関数 kuku は九九の問題を１問出し、答えを待つ。
// 正答、誤答の別を表示する。
// 正答なら 1 を返し、誤答なら 0 を返す。
int kuku(int qn)
{
    int x, y, result;
    char buffer[BUFFER_SIZE];

    // 問題をランダムに生成する。
    x = rand() % 9 + 1;
    y = rand() % 9 + 1;

    // 出題する。
    printf("［第 %d 問］　%d × %d ＝", qn + 1, x, y);

    // 端末が改行待ちで表示を抑止するのを防ぐ。
    fflush(stdout);

    // 解答の入力を待つ。
    get_line(buffer, BUFFER_SIZE);

    // 解答を整数に変換する。
    result = atoi(buffer);

    // 正答か誤答か確認する。
    if (x * y == result) {
        printf("はい、正しいです。\n");
        return 1;
    } else {
        printf("残念、まちがいです。\n");
        return 0;
    }
}

void get_line(char *buffer, int size)
{
    if (fgets(buffer, size, stdin) == NULL) {
        buffer[0] = '\0';
        return;
    }

    for (int i = 0; i < size; i++) {
        if (buffer[i] == '\n') {
            buffer[i] = '\0';
            return;
        }
    }
}
